from gv33 import gv33


class BitArray:
    def __init__(self, size):
        self.size = size
        self.bytearray = bytearray((size + 7) >> 3)

    def clear(self):
        ba = self.bytearray
        for i in range(len(ba)):
            ba[i] = 0

    def get_bit(self, bit_ix):
        if not isinstance(bit_ix, int):
            raise IndexError("bit array index not an int")
        
        if bit_ix < 0:
            bit_ix = self.size + bit_ix

        if bit_ix >= self.size:
            raise IndexError("bit array index out of range")

        byte_ix = bit_ix >> 3
        bit_ix = bit_ix & 7
        return (self.bytearray[byte_ix] >> bit_ix) & 1

    def set_bit(self, bit_ix, val):
        if not isinstance(bit_ix, int):
            raise IndexError("bit array index not an int")
        
        if bit_ix < 0:
            bit_ix = self.size + bit_ix

        if bit_ix >= self.size:
            raise IndexError("bit array index out of range")

        if not isinstance(val, int):
            raise ValueError("bit array value not an int")

        if val not in (0, 1):
            raise ValueError("bit array value must be 0 or 1")

        byte_ix = bit_ix >> 3
        bit_ix = bit_ix & 7
        bit_val = 1 << bit_ix

        if val:
            self.bytearray[byte_ix] |= bit_val
        else:
            self.bytearray[byte_ix] &= ~bit_val

    def __getitem__(self, key):
        return self.get_bit(key)

    def __setitem__(self, key, value):
        self.set_bit(key, value)

#die kleinen panels haben ein anderes mapping als die großen, deshalb hier eine übersetzung
translation = {
    20:1,
    25:2,
    24:3,
    23:4,
    21:5,
    26:6,
    27:7,
    22:8,
    29:9,
    30:10,
    31:11,
    28:12,
    32:13,
    34:14,
    35:15,
    38:16,
    37:17,
    7:18,
    36:19,
    1:20,
    4:21,
    5:22,
    3:23,
    2:24,
    6:25,
    33:26,
    8:27,
    9:28,
    10:29,
    11:30,
    12:31,
    13:32,
    18:33,
    14:34,
    19:35,
    17:36,
    16:37,
    15:38,
}

def get_bytes(char, inverted = False):
    
    # 5 Byte pro Display
    # MSB = Segment enable bit
    # LSB = Dimmer ? immer auf 1 setzen!


    byts = BitArray(5*8)

    # set MSB and LSB
    byts[0]=True
    byts[-1]=True

    # fill array
    for i in range(byts.size)[1:-1]:
        byts[i]=not inverted

    
    # if invalid char, send blank
    if not gv33.get(char):
        return byts.bytearray
    
    # set segments based on mapping
    for segment in gv33.get(char):
        segment = translation[segment]

        if segment > 38 or segment < 1:
            continue

        byts[-segment-1]=inverted

    return reversed(byts.bytearray)